function addHandler(obj, evnt, handler) {
    if (obj.addEventListener) {
        obj.addEventListener(evnt.replace(/^on/, ''), handler, false);
    } else {
        if (obj[evnt]) {
            var origHandler = obj[evnt];
            obj[evnt] = function(evt) {
                origHandler(evt);
                handler(evt);
            }
        } else {
            obj[evnt] = function(evt) {
                handler(evt);
            }
        }
    }
}
addHandler(window, 'onerror', function (msg, url, num) {
    var msgW = XPCNativeWrapper.unwrap(msg);

    log("Background window error: " + msgW + ';' + url + ';' + num);
    return true;
});


var logEnabled = false;
function log(message) {
    if (logEnabled) {
        console.log(message);
    }
}


var localAudioPlayer = {

    _invoke: function(code) {
        var rnd = Math.floor(Math.random()*1000);
        var id = "utl_"+rnd;
        var $t = $("<div id='"+id+"' onclick='"+code+"'></div>");
        $("body").append($t);
        $("#"+id).click();
        setTimeout(function() {
            $("#"+id).remove()
        }, 3000)

    },

    play: function() {
//        $("#ac_play").click();
        this._invoke("if (audioPlayer.player) audioPlayer.player.playAudio();");
    },

    pause: function() {
//        $("#ac_play").click();
        this._invoke("if (audioPlayer.player) audioPlayer.player.pauseAudio();");
    },

    seek: function(song_id, pos) {
//        var $t = $("<div id='"+id+"' onclick='audioPlayer.prStep = 1; audioPlayer.changePr("+parseInt(pos)+");'></div>");
        this._invoke("if (audioPlayer.player) audioPlayer.player.playAudio("+parseInt(pos)+");");
    },

    setFile: function (song_id, songInfo) {

        var rnd = Math.floor(Math.random()*1000);
        var id = "utl_"+rnd;

        log("Playing: "+song_id+", div id="+id);

        var $t;
        if (songInfo) {

            var songInfo = JSON.stringify([songInfo.pid, songInfo.id, songInfo.url, songInfo.duration])

            var cr = "<div id='"+id+"' onclick='playAudioNew(\""+song_id+"\", null, {songInfo:"+songInfo+"});'></div>"
            log("setFile: "+cr);
            $t = $(cr);
        }
        else {
            $t = $("<div id='"+id+"' onclick='playAudioNew(\""+song_id+"\");'></div>");
        }
        $("body").append($t);
        $("#"+id).click();
        setTimeout(function() {
            $("#"+id).remove()
        }, 3000)
    },


    setVolume : function(vol) {
        var rnd = Math.floor(Math.random()*1000);
        var id = "utl_"+rnd;
        var $t = $("<div id='"+id+"' onclick='if (audioPlayer.player) audioPlayer.player.setVolume("+vol+");'></div>");
        $("body").append($t);
        $("#"+id).click();
        setTimeout(function() {
            $("#"+id).remove()
        }, 3000)

    },

    requestMode : function() {
        var rnd = Math.floor(Math.random()*1000);
        var id = "utl_"+rnd;
        var $t = $("<div id='"+id+"' onclick='if (audioPlayer.player) window.postMessage({action:\"onMode\", param:audioPlayer.player.paused()}, \"*\");'></div>");
        $("body").append($t);
        $("#"+id).click();
        setTimeout(function() {
            $("#"+id).remove()
        }, 3000)

    },

    onError: function (e) {
        self.port.emit("log", {msg: "mode: error "+e});
    },

    onPlayProgress: function(e) {
        self.port.emit("vk_player", {action:'onPlayProgress', data: e});
    },

    onLoadComplete: function(e) {
        self.port.emit("vk_player", {action:'onLoadComplete', data: e});
    },

    onPlayFinish: function(e) {
        self.port.emit("vk_player", {action:'onPlayFinish'});
    },

    onLoadProgress: function(e) {
        self.port.emit("vk_player", {action:'onLoadProgress', data: e});
    }

}

// listen to events from 'page script': https://addons.mozilla.org/en-US/developers/docs/sdk/latest/dev-guide/guides/content-scripts/communicating-with-other-scripts.html
document.defaultView.addEventListener('message', function(event) {
    switch (event.data.action) {
        case "onLoadProgress":
            localAudioPlayer.onLoadProgress(event.data.param);
            break;
        case "onLoadComplete":
            localAudioPlayer.onLoadComplete(event.data.param);
            break;
        case "onPlayProgress":
            localAudioPlayer.onPlayProgress(event.data.param);
            break;
        case "onPlayFinish":
            localAudioPlayer.onPlayFinish(event.data.param);
            break;
        case "onError":
            localAudioPlayer.onError(event.data.param);
            break;

        case "onMode":
            self.port.emit('vk_player', {action: "set_mode", mode: event.data.param===false ? 'play' : 'paused' })
            break;
    }
}, false);


self.port.on("vk_player", function (cmd) {
    if (cmd.action == 'play') {
        localAudioPlayer.play();
    }
    else if (cmd.action == 'pause') {
        localAudioPlayer.pause();
    }
    else if (cmd.action == 'set') {
        localAudioPlayer.setFile(cmd.id, cmd.song);
    }
    else if (cmd.action == 'seek') {
        localAudioPlayer.seek(cmd.id, cmd.pos);
    }
    else if (cmd.action == 'request_mode') {
        localAudioPlayer.requestMode();
    }
    else if (cmd.action == 'set_volume') {
        localAudioPlayer.setVolume(cmd.volume);
    }
    else {
        localAudioPlayer.play();
    }
});

$(function() {


    log("Creating proxy")
    function proxy(method, skip) {

        return "(function(m, s) {if (!window.audioPlayer) { return; } var orig = window.audioPlayer[m]; " +
                "window.audioPlayer[m] = function(e) {" +
                "window.postMessage({action:m, param:e}, \"*\"); " +
                "if (!s) {orig();} " +
            "}})(\""+method+"\", "+skip+");";
    }


    var installCode = "try {"+ proxy('onLoadComplete') + proxy('onLoadProgress') + proxy('onPlayFinish', true) + proxy('onPlayProgress') + proxy('onError')+"} catch(error) { log(error); }";
    log("Install code: "+installCode)

    var $div = $("<div onclick='" +installCode + " setInterval(function() {if (audioPlayer.player) window.postMessage({action:\"onMode\", param:audioPlayer.player.paused()}, \"*\")}, 250)'></div>");
    $("body").append($div);
    $div.click();

    log("Created proxy")


});

