function ToolbarUI(platformAdapter) {


    this.platformAdapter = platformAdapter;

    this.displayLeft = false;
    this.currentDuration = 0;
    var currentVolume = 0;
    this.currentSeekPosition = 0;
    this.appendCallbackInProgress = false;

    this.lastPlaylistId = null;

    var self = this;

    this.messageListener = function(msg) {

        if (msg.context == "toolbar_ui") {

            switch (msg.action) {
                case 'set_model':

                    var model = msg.data;

                    self.setMode(model.mode)

                    if (model.authenticated) {
                        self.hideLogin();
                    }
                    else {
                        self.showLogin();
                    }

                    if (model.playlist) {
                        self.populatePlaylist(model.playlist, model.playlist_id);
                    }
                    self.setCurrentSong(model.current)
                    if (model.current) {
                        self.setDuration(model.current.duration)
                    }
                    else {
                        self.updatePlayed(0)
                    }
                    self.setVolume(model.volume)
                    self.setShuffle(model.shuffle)
                    self.setRepeat(model.repeat)
                    if (model.playlist_id=='search') {
                        self.setSearchString(model.lastQuery)
                    }
                    self.setLastFMTitle(model.enableLastFm)
                    break;

                case 'populate_playlist':
                    self.populatePlaylist(msg.data.playlist, msg.data.playlist_id, msg.data.offset);
                    break;

                case 'set_authenticated':
                    if (msg.data) {
                        self.hideLogin();
                    }
                    else {
                        self.showLogin();
                    }
                    break;

                case 'set_current_song':
                    self.setCurrentSong(msg.data.song)
                    break;

                case 'set_repeat':
                    self.setRepeat(msg.data)
                    break;

                case 'set_shuffle':
                    self.setShuffle(msg.data)
                    break;

                case 'set_duration':
                    self.setDuration(msg.data.duration)
                    break;

                case 'set_mode':
                    self.setMode(msg.data.mode)
                    break;

                case 'update_played':
                    self.updatePlayed(msg.data)
                    break;

                case 'populate_video':
                    self.populateVideo(msg.data.playlist, msg.data.append)
                    break;

                case 'add_history_item':
                    self.addHistoryItem(msg.data);
                    break;

                case 'populate_history':
                    self.populateHistory(msg.data);
                    break;

                case 'toggle_lastfm':
                    self.setLastFMTitle(msg.data);
                    break;

            }

        }
    };

}

ToolbarUI.prototype.getMessageListener = function() {
    return this.messageListener;
};

ToolbarUI.prototype.bind = function() {

    log("Binding toolbar UI")
    var self = this;

    const $audio = $("#playlist");
    $audio.empty()


    $('.lastfm').on('click', function () {
        self.platformAdapter.sendMessage({context: "vk_player", action: "toggle_lastfm" })
    });
    var $tabs = $("#tabs");
    $tabs.tabs({
        active: 0,
        activate: function(event, ui) {
            if (ui.newPanel.attr("id")==="watch") {
                setTimeout(function() {$("#video-search-query").focus();}, 200)

                self.platformAdapter.sendMessage({context: "vk_player", action: "load_video_playlist"})

            }
        }
    });

    $('#share-subpanel').tabs({ active: 0 });
    self.platformAdapter.sendMessage({context: "vk_player", action: "get_history", data: 'populate_history' })
 
    // move the nav to the bottom
    $( ".tabs-bottom .ui-tabs-nav" ).appendTo( ".tabs-bottom" );

    $("#volume-slider").slider({
        min: 0,
        max: 100,
        slide: function( event, ui ) {
            var val = ui.value;
            self.platformAdapter.sendMessage({context: "vk_player", action: "set_volume", data: val / 100.0});
        },
        change: function( event, ui ) {
            var val = ui.value;
            self.platformAdapter.sendMessage({context: "vk_player", action: "set_volume", data: val / 100.0});
        }
    });

    const $mode = $("#mode-panel");
    const $share = $("#share-subpanel");
    $mode.find("li").click(function() {
        $mode.find("li").removeClass("active");
        $(this).addClass("active");
    });


    $("#play").click(function(e) {
        self.platformAdapter.sendMessage({context: "vk_player", action: "toggle"});
    });

    $("#next").click(function () {
        self.platformAdapter.sendMessage({context: "vk_player", action: "next_song"});
    });

    $("#prev").click(function () {
        self.platformAdapter.sendMessage({context: "vk_player", action: "prev_song"});
    });

    $mode.find(".my-playlist").click(function () {
        self.platformAdapter.sendMessage({context: "vk_player", action: "restore_playlist"});
    });

    $mode.find(".recommendations").click(function () {
        self.platformAdapter.sendMessage({context: "vk_player", action: "load_recommendations"});
    });

    $("#shuffle").click(function () {
        self.platformAdapter.sendMessage({context: "vk_player", action: "toggle_shuffle"});
    });

    $("#repeat").click(function () {
        self.platformAdapter.sendMessage({context: "vk_player", action: "toggle_repeat"});
    });

    $("#download-all-btn").click(function() {
        self.platformAdapter.sendMessage({context: "vk_player", action: "download-all"});
    });


    $("#progress-panel, #progress").click(function (e) {
        var $this = $("#progress-panel");
        var posX = $this.offset().left;
        var duration = (e.pageX - posX) / ( $this.outerWidth() - 2);
        log("Seeking to percent: " + duration + ", pos: " + (duration * self.currentDuration) + ", posX=" + posX + ", pageX=" + e.pageX + ", w=" + $this.width())

        self.platformAdapter.sendMessage({context: "vk_player", action: "seek", data: duration * self.currentDuration});

    });

    $("#search-form").submit(function(e) {

        var query = $("#search-query").val();
        if (!query) {
            e.preventDefault()
            return false;
        }
        self.platformAdapter.sendMessage({context: "vk_player", action: "search", data: {query: query}});

        e.preventDefault();
        return false;
    });

    $("#playlist-wrapper").scroll(function(e) {
        const height = $(this).find("#playlist").height();
        const height2 = $(this).height();
        var scrolledToEnd = this.scrollTop >= (height - height2 - 23*2);
        if (scrolledToEnd) {
            log("Scrolled to the end")
            if (self.appendCallbackInProgress) {
                log("Append in progress")
                return
            }
            self.appendCallbackInProgress = true;
            log("Loading next portion of tracks for "+self.lastPlaylistId)
            switch (self.lastPlaylistId) {
                case 'recommendations' :
                    self.platformAdapter.sendMessage({context: "vk_player", action: "load_recommendations", data: {offset: -1, append: true}});
                    break;
                case 'search' :
                    self.platformAdapter.sendMessage({context: "vk_player", action: "search", data: {query: null, offset: -1}});
                    break;

            }

        }
    });

    const $video = $("#video-playlist");
    $video.empty();

    $("#video-search-form").submit(function(e) {
        const query = $("#video-search-query").val();
        log("Searching videos: "+query)
        self.platformAdapter.sendMessage({context: "vk_player", action: "search_videos", data: {query: query, offset: 0}});
        e.preventDefault();
    });

    $("#video-results").scroll(function(e) {
        const $this = $(this);
        const $videoPlaylist = $this.find("#video-playlist");
        const height = $videoPlaylist.height();
        const height2 = $this.height();
        var scrolledToEnd = this.scrollTop >= (height - height2);
        log("scrollTop="+this.scrollTop+", playlist height="+height+", this.height="+height2)

        const size = $videoPlaylist.find("li").size();

        if (scrolledToEnd) {
            if (self.appendCallbackInProgress) {
                return
            }
            self.appendCallbackInProgress = true;
            log("Appending videos. Offset="+size)

            self.platformAdapter.sendMessage({context: "vk_player", action: "search_videos", data: {query: null, offset: size}});

        }
    });



    $('#playlist-wrapper').tooltip();
    $('.lastfm').tooltip();


    share.init(this.platformAdapter);
    log("Binded toolbar UI");



};

ToolbarUI.prototype.populateVideo = function(videos, append) {
    log("Populating videos. Append="+append)
    const $video = $("#video-playlist");
    if (!append) {
        $video.empty();
    }
    for (var i = 0; i < videos.length; i++) {
        var video = videos[i];
        var $li = this.appendVideo(video, $video);
    }

    this.appendCallbackInProgress = false;

};

ToolbarUI.prototype.appendVideo = function(video, $video) {

    var self = this;

    var $li = $("<li/>", {
        class: video.id
    })
        .click(function () {
            self.platformAdapter.openWindow({
                url: "http://vk.com/video"+video.id+"_"+video.pid,
                width: 1024,
                height: 768
            })
        })
        .appendTo($video);

    $("<img/>", {
        src: video.url
    }).appendTo($li);
    $("<p/>", {
        html: video.desc
    }).appendTo($li);
    return $li;
};

ToolbarUI.prototype.setMode = function(mode) {
    if (mode=='play') {
        $("#play").removeClass("pause")
    }
    else {
        $("#play").addClass("pause")
    }

};

ToolbarUI.prototype.setRepeat = function(repeat) {
    if (repeat) {
        $("#repeat").addClass("active")
    }
    else {
        $("#repeat").removeClass("active")
    }
};

ToolbarUI.prototype.setSearchString = function(query) {
    $("#search-query").val(query)
};

ToolbarUI.prototype.showLogin = function() {
    $("#login-panel").show();
};

ToolbarUI.prototype.hideLogin = function() {
    $("#login-panel").hide();
};


ToolbarUI.prototype.setShuffle = function(shuffle) {
    if (shuffle) {
        $("#shuffle").addClass("active")
    }
    else {
        $("#shuffle").removeClass("active")
    }
};

ToolbarUI.prototype.updateCurrentPos = function(currentPos) {
    this.currentSeekPosition = currentPos;
    var percent = currentPos / (this.currentDuration);
    if (percent>1) {
        percent = 1;
    }
    return percent;
};

ToolbarUI.prototype.setVolume = function(volume) {
    $( "#volume-slider" ).slider( "value", volume * 100.0 );
};

ToolbarUI.prototype.updatePlayed = function(currentPos) {

    if (this.currentDuration <= 0) {
        $("#progress").width(0);
    }
    else {
        var percent = this.updateCurrentPos(currentPos);
        var progressPanelWidth = $("#progress-panel").outerWidth() - 2; //2 = border
        var pos = percent * progressPanelWidth;
        if (pos>progressPanelWidth) {
            pos = progressPanelWidth;
        }
        $("#progress").width(pos);
    }


    function formatTime(time) {

        var min = Math.floor(time / 60);
        var sec = Math.floor(time % 60);

        var minstr = min + "";
        if (min<10) {
            minstr = "0" + min;
        }

        var secstr = sec + "";
        if (sec<10) {
            secstr = "0" + sec;
        }

        return {
            min: minstr,
            sec: secstr
        }
    }
    var played = formatTime(this.displayLeft ? this.currentDuration - currentPos : currentPos);
    var total = formatTime(this.currentDuration);


    $("#progress-panel").find(".time-elapsed").html((this.displayLeft ? "- " : "") + played.min + ":" + played.sec);
};

ToolbarUI.prototype.setCurrentSong =  function(song) {
    var $playlist = $("#playlist-wrapper");
    $playlist.find("li.active").removeClass("active");

    if (!song) {
        return
    }
    $("#progress-panel").find(".time-total").html(song.duration);
    $("#track-title").html("<span class='artist'>"+song.artist + "</span> - <span class='title'>" + song.name +"</span>")

    var songLi = $playlist.find("li." + song.id);

    if (songLi.size()<=0) {
        log("Song "+song.id+" not found in list")
        return
    }



    var songLiPrev = songLi.prev();
    songLi.addClass("active");
    $playlist.scrollTo(songLiPrev.size()>0 ? songLiPrev.get(0) : songLi.get(0), 200);
};

ToolbarUI.prototype.setDuration = function(durationStr) {
    var tokens = durationStr.split(":");
    var min = tokens[0];
    var sec = tokens[1];
    var durationsInSec = parseInt(min) * 60 + parseInt(sec);

    this.currentDuration = durationsInSec;
    log("Popup page set current duration to " + this.currentDuration + " (" + durationStr + ", min=" + min + ", sec=" + sec + ")")

};

ToolbarUI.prototype.populatePlaylist = function(playlist, playlistId, offset) {

    var self = this;

    if (!offset) {
        offset = 0;
    }

    this.lastPlaylistId = playlistId;


    var newPlaylist = offset==0;

    var $playlist = $("#playlist");
    if (newPlaylist) {
        $playlist.empty().scrollTo(0);
    }

    var last = $playlist.find("li:last").attr("class");


    const $mode = $("#mode-panel");
    $mode.find("li").removeClass("active");
    if (playlistId=='recommendations') {
        $mode.find(".recommendations").addClass("active")
    }
    else if (playlistId=='playlist') {
        $mode.find(".my-playlist").addClass("active")
    }


    for (var index = offset; index < playlist.songs.length; index++) {

        var song = playlist.songs[index];
        (function(song) {
            var $li = $("<li/>").addClass(song.id);
            $li.appendTo($playlist);

            //song name
            var $nameWrapper = $("<span/>", {
                class: "name-wrapper",
                title: self.decodeEntities(song.artist + " - " + song.name),
                html: "<span class='artist'>"+song.artist + "</span> - <span class='title'>" + song.name +"</span>"
            }).appendTo($li);


            $("<span/>", {
                class: "fadeout"
            }).appendTo($nameWrapper);



            //actions
            var $actions = $("<span/>", {
                class: "actions"
            }).appendTo($li);


            //time
            $("<span/>", {
                class: "time",
                text: song.duration
            }).appendTo($li);

            var $added = $("<span/>", {class: "added"});




            //hide recommendation
            if (song.hashAdd) {
                $("<span/>", {
                    class: "add",
                    title: "Добавить в плейлист"
                }).click(function (e) {
                        self.platformAdapter.sendMessage({context: "vk_player", action: "add_share", data: song})

                        $( ".ui-tooltip" ).remove()
                        $(this).replaceWith($added);
                        e.stopPropagation()
                    })
                    .appendTo($actions);
            }
            if (song.hashHide) {
                $("<span/>", {
                    title: "Удалить и больше не показывать",
                    class: "hide"
                }).click(function (e) {
                        self.platformAdapter.sendMessage({context: "vk_player", action: "hide_recommendations", data: song})
                        $( ".ui-tooltip" ).remove()
                        $li.remove();
                        e.stopPropagation()
                    })
                    .appendTo($actions);
            }

            if (song.from_playlist) {
                $("<span/>", {
                    title: "Сохранить",
                    class: "download"
                }).click(function (e) {
                        self.platformAdapter.sendMessage({context: "vk_player", action: "download", data: song});
                        e.stopPropagation()
                    })
                    .appendTo($actions);

                $("<span/>", {
                    title: "Удалить из плейлиста",
                    class: "delete"
                }).click(function (e) {
                        self.platformAdapter.sendMessage({context: "vk_player", action: "delete_audio", data: song})
                        $( ".ui-tooltip" ).remove()
                        $li.remove();
                        e.stopPropagation()
                    })
                    .appendTo($actions);

            }


            var changeSong = function () {
//            setDuration(song.duration)
//            setCurrentSong(song)
                log("Selected song: "+song)
                self.platformAdapter.sendMessage({context: "vk_player", action: "set_song", data: song})
            };
            $li.click(changeSong);

        })(song)

    }


    if (playlist.songs.length == 0) {
        var noSongsText;
        if (playlistId == 'search') {
            noSongsText = "Ничего не найдено";
        }
        else {
            noSongsText = "Список пуст";
        }

        var $li = $("<li/>", {
            text: noSongsText
        });
        $li.appendTo($playlist);

        $("#download-all-btn").hide()

    }
    else {
        if (playlistId=="playlist") {
            $("#download-all-btn").show();
        }
        else {
            $("#download-all-btn").hide();
        }

    }




    $( ".ui-tooltip" ).remove()

    this.appendCallbackInProgress = false;


};

ToolbarUI.prototype.addHistoryItem = function(item) {

    var item
        , container = $('#share-history ul');
    
    container.prepend('<li><div class="shared-page"><img class="history-icon" src="' + item[0] +
        '"/><a class="history-link" href="' + item[2] + '">' + item[1] +
        '</a></div><div class="shared-sn"><span class="img ' + item[3] + '-sn"/><span> ' + item[4] + '</div></li>');
};

ToolbarUI.prototype.populateHistory = function(history) {

    var item
        , self = this
        , length = history.length
        , container = $('#share-history ul');

    container.on('click', 'a', function (e) {
        e.preventDefault();
        self.platformAdapter.openLink($(this).attr('href'));
    });

    while(length--) {
        item = history[length];
        container.append('<li><div class="shared-page"><img class="history-icon" src="' + item[0] +
            '"/><a class="history-link" href="' + item[2] + '">' + item[1] +
            '</a></div><div class="shared-sn"><span class="img ' + item[3] + '-sn"/></span> ' + item[4] + '</div></li>');
    }
};


ToolbarUI.prototype.decodeEntities = (function() {
    // this prevents any overhead from creating the object each time
    var element = document.createElement('div');

    function decodeHTMLEntities (str) {
        if(str && typeof str === 'string') {
            // strip script/html tags
            str = str.replace(/<script[^>]*>([\S\s]*?)<\/script>/gmi, '');
            str = str.replace(/<\/?\w(?:[^"'>]|"[^"]*"|'[^']*')*>/gmi, '');
            element.innerHTML = str;
            str = element.textContent;
            element.textContent = '';
        }

        return str;
    }

    return decodeHTMLEntities;
})();

ToolbarUI.prototype.setLastFMTitle = function(enable) {
    $('.lastfm').tooltip('option', {
        content: enable ? 'Выключить скробблинг в LastFM' : 'Включить скробблинг в LastFM'
    });
    $('.lastfm').css('background-image', 'url("img/lastfm' + (enable ? '' : '_disabled') + '.png")');
};

ToolbarUI.prototype.removeSymbols = (function() {

    function removeSymbols (str) {
        if(str && typeof str === 'string') {
            str = str.replace(/[/()\\&;.,#'?<>*:|]/g, "_")
        }

        return str;
    }

    return removeSymbols;
})();

var logEnabled = false;
function log(message) {
    if (logEnabled) {
        log(message);
    }
}
