
function Toolbar(platformAdapter, options) {

    options = options || {};

    var defaultSettings = {
        lastfmEnabled : false,
        repeat: true,
        shuffle: false,
        volume: 1.0
    };

    this.settings = extend(defaultSettings, options);



    this.mode = "pause";
    this.authenticated = false;
    this.platform = platformAdapter;

    var self = this;

    var toolbar = this;

    this.logger = new Logger(platformAdapter, this);



    this.listener = function(msg) {
        if (msg.context == "vk_player") {

            switch (msg.action) {
                case 'toggle':
                    self.toggle();
                    break;

                case 'next_song':
                    self.nextSong();
                    break;

                case 'set_song':
                    self.setSong(msg.data);
                    break;

                case 'prev_song':
                    self.prevSong();
                    break;

                case 'share_window_opened':
                    self.platform.prepareShare(msg.data.type, msg.data.data, msg.data.id);
                    break;

                case 'get_history':
                    self.getShareHistory();
                    break;

                case 'restore_playlist':
                    self.restorePlaylist();
                    break;

                case 'load_recommendations':
                    self.loadRecomendations(msg.data ? msg.data.offset : null, msg.data ? msg.data.append : false);
                    break;

                case 'toggle_shuffle':
                    self.toggleShuffle();
                    break;

                case 'toggle_repeat':
                    self.toggleRepeat();
                    break;

                case 'set_volume':
                    self.setVolume(msg.data);
                    break;

                case 'seek':
                    self.seek(msg.data);
                    break;

                case 'add_share':
                    self.addShareAudio(msg.data);
                    break;

                case 'hide_recommendations':
                    self.hideRecommendations(msg.data);
                    break;

                case 'delete_audio':
                    self.deleteAudio(msg.data);
                    break;

                case 'request_model':
                    self.requestModel();
                    break;

                case 'load_video_playlist':
                    self.loadVideoPlaylist(msg.data);
                    break;

                case 'search':
                    self.search(msg.data.query, msg.data.offset);
                    break;

                case 'search_videos':
                    self.loadVideos(msg.data.query, msg.data.offset);
                    break;

                case 'lastfm_callback':
                    self.lastFM.notify(msg.data);
                    break;

                case 'download':
                    self.download(msg.data);
                    break;

                case 'toggle_lastfm':
                    self.toggleLastFM(msg.data);
                    break;

                case 'download-all':
                    self.downloadAll();
                    break;

            }

        }
    };


    this.lastFM = {

        api_key:"c628f07ac038c465a74c3d51c5890419",
        api_secret:"81758af0b9fba081c05b46106b0bc47a",
        wndIds : [],

        init: function() {

            var self = this;

            platformAdapter.storage.get("last_fm_session_key", function(value) {
                self.sessionKey = value;
            });
            platformAdapter.storage.get("last_fm_token", function(value) {
                self.token = value;
            });

        },


        auth: function(callback) {

            var fm = this
                , args = [].slice.call(arguments, 1);

            if (!this.token) {
                log("No last.fm token found. Requesting new")
                platformAdapter.openWindow({
                    url: 'http://www.last.fm/api/auth?api_key=' + fm.api_key + '&cb='+platformAdapter.getURL('lastfm_cb.html'),
                    width: 1024,
                    height: 700,
                    type: 'popup'
                }, function(wnd) {
                    if (wnd) {
                        fm.wndIds.push(wnd)
                    }
                });
                return;
            }

            var params = {
                api_key : this.api_key,
                method: "auth.getSession",
                token: this.token
            };

            params.api_sig = this.getApiSignature(params);
            params.format="json";


            platformAdapter.ajax({
                url: 'http://ws.audioscrobbler.com/2.0/',
                type: "GET",
                data: params,
                success: function(res) {
                    if (!res) {
                        return
                    }
                    if (res.error == 4) {
                        platformAdapter.openWindow({
                            url: 'http://www.last.fm/api/auth?api_key=' + fm.api_key + '&cb='+platformAdapter.getURL('/lastfm_cb.html'),
                            width: 1024,
                            height: 700,
                            type: 'popup'
                        }, function(wnd) {
                            fm.wndIds.push(wnd)
                        });
                        return
                    }

                    if (res.error) {
                        log("Unknown error while requesting last.fm: "+res.error);
                        return;
                    }

                    fm.sessionKey = res.session.key;
                    platformAdapter.storage.put("last_fm_session_key", fm.sessionKey);
                    callback.apply(fm, args);

                },
                error: function() {
                    log("Error while commited scrobble into last.fm")
                }

            });

        },

        notify: function(url) {
            var uri = parse_url(url);
            var vars = uri.query.split('&');

            var opts = {}

            for (var i = 0; i < vars.length; i++) {
                var pair = vars[i].split('=');
                opts[pair[0]] = pair[1];
            }

            this.token = opts["token"];
            platformAdapter.storage.put("last_fm_token", this.token);

            this.auth(function() {
                log("Authenticated to last fm")
            });

            for (var j = 0; j < this.wndIds.length; j++) {
                platformAdapter.closeWindow(this.wndIds[j]);
            }
            this.wndIds = [];

        },

        scrobble : function(song, startPlaying) {

            if (!toolbar.settings.enableLastFm) {
                return
            }


            var self = this;

            if (!this.sessionKey) {
                this.auth(function(){this.scrobble(song)});
                return;
            }

            var tokens = song.duration.split(":");
            var min = tokens[0];
            var sec = tokens[1];
            var durationsInSec = parseInt(min) * 60 + parseInt(sec);


            var params = {
                api_key : this.api_key,
                method: "track.scrobble",
                artist: html_entity_decode(song.artist),
                track: html_entity_decode(song.name),
                sk : this.sessionKey,
                duration: durationsInSec,
                timestamp : Math.round(startPlaying.getTime() / 1000)
            };

            var sig = this.getApiSignature(params);
            params.api_key = this.api_key;
            params.api_sig = sig;
            params.sk = this.sessionKey;
            params.format="json";


            platformAdapter.ajax({
                url: 'http://ws.audioscrobbler.com/2.0/',
                type: "POST",
                data: params,
                success: function(resp) {
                    if (!resp) {
                        return
                    }
                    if (typeof resp == 'string') {
                        resp = JSON.parse(resp);
                    }
                    if (resp.error == 4 || resp.error==9) {
                        self.auth(self.scrobble, song, startPlaying)
                        return
                    }
                    log("Commited scrobble into last.fm")
                },
                error: function() {
                    log("Error while commited scrobble into last.fm")
                }

            });


        },

        updateNowPlaying : function(song) {

            if (!toolbar.settings.enableLastFm) {
                return
            }

            if (!this.sessionKey) {
                this.auth(function(){this.updateNowPlaying(song)});
                return;
            }
            var tokens = song.duration.split(":");
            var min = tokens[0];
            var sec = tokens[1];
            var durationsInSec = parseInt(min) * 60 + parseInt(sec);
            var self = this;



            var params = {
                api_key : this.api_key,
                method: "track.updateNowPlaying",
                artist: html_entity_decode(song.artist),
                track: html_entity_decode(song.name),
                sk : this.sessionKey,
                duration: durationsInSec
            };

            var sig = this.getApiSignature(params);
            params.api_sig = sig;
            params.format="json";


            platformAdapter.ajax({
                url: 'http://ws.audioscrobbler.com/2.0/',
                type: "POST",
                data: params,
                success: function(resp) {
                    if (!resp) {
                        return
                    }
                    if (typeof resp == 'string') {
                        resp = JSON.parse(resp);
                    }
                    if (resp.error == 4 || resp.error==9) {
                        self.auth(self.updateNowPlaying, song)
                        return
                    }
                    log("Commited updateNowPlaying into last.fm")
                },
                error: function() {
                    log("Error while commited updateNowPlaying into last.fm")
                }

            });


        },

        getApiSignature : function(params){
            var keys   = [];
            var string = '';

            for(var key in params){
                keys.push(key);
            }

            keys.sort();

            for(var index in keys){
                var key = keys[index];

                string += key + params[key];
            }

            string += this.api_secret;

//            log("Signing string: "+string)

            /* Needs lastfm.api.md5.js. */
            return md5(string);
        }

    }

    this.lastFM.init();

}

Toolbar.prototype.getShareHistory = function() {
    this.platform.getHistory([], 0);
};

Toolbar.prototype.setEnableLastFM = function(enable) {
    this.settings.enableLastFm = enable;
};

Toolbar.prototype.setAuthenticated = function(authenticated) {
    this.authenticated = authenticated;
};

Toolbar.prototype.download = function(song) {
    this.platform.download(song);

};

Toolbar.prototype.downloadAll = function() {
    this.platform.downloadAll(this.playlist.songs);
};

Toolbar.prototype.requestModel = function() {
    var model = {
        mode: this.mode,
        playlist: this.playlist,
        playlist_id: this.playlist_id,
        current: this.current,
        volume: this.settings.volume,
        shuffle: this.settings.shuffle,
        repeat: this.settings.repeat,
        enableLastFm: this.settings.enableLastFm,
        authenticated: this.authenticated
    };

    this.platform.sendMessage({context: "toolbar_ui", action: "set_model", data: model});

};

Toolbar.prototype.setVKUserId = function(id) {
    this.vk_user_id = id;
};

Toolbar.prototype.play = function() {

    if (!this.current && this.playlist.songs) {
        this.setSong(this.playlist.songs[0])
    }

    this.platform.player.play();
    this.setMode("play");
};

Toolbar.prototype.pause = function() {
    this.platform.player.pause();
    this.setMode("pause");
};

Toolbar.prototype.setMode = function(mode) {
    log("Mode: "+mode);
    this.mode = mode;
    this.platform.sendMessage({context: "toolbar_ui", action: "set_mode", data: { mode: this.mode } } );

};

Toolbar.prototype.songEnded = function() {
    this.lastFM.scrobble(this.current, this.startPlaying)
    this.nextSong();
};

Toolbar.prototype.toggle = function() {
    if (this.isPaused()) {
        this.play();
    }
    else {
        this.pause();
    }
};

Toolbar.prototype.isPaused = function() {
    return this.mode === "pause";
//    return $player.data().jPlayer.status.paused;
};

Toolbar.prototype.toggleShuffle = function() {
    this.settings.shuffle = !this.settings.shuffle;
    this.platform.storage.put("toolbar_settings", this.settings);
    this.platform.sendMessage({context: "toolbar_ui", action: "set_shuffle", data: this.settings.shuffle } );
};

Toolbar.prototype.toggleRepeat = function() {
    this.settings.repeat = !this.settings.repeat;
    this.platform.storage.put("toolbar_settings", this.settings);
    this.platform.sendMessage({context: "toolbar_ui", action: "set_repeat", data: this.settings.repeat } );
};

Toolbar.prototype.setSong = function(song) {
    this.current = song;
    this.current_pos = 0;
    this.startPlaying = new Date();
    this.platform.player.setSong(song);

    this.mode = "play";

    this.platform.sendMessage({ context: "toolbar_ui", action: "set_current_song", data: { song: song } } );
    this.platform.sendMessage({ context: "toolbar_ui", action: "set_duration", data: { duration: song.duration } } );
    this.platform.sendMessage({ context: "toolbar_ui", action: "set_mode", data: { mode: this.mode } } );

    this.lastFM.updateNowPlaying(song);

};

Toolbar.prototype.seek = function(pos) {
    this.platform.player.seek(pos);
};

Toolbar.prototype.setVolume = function(volume) {
    this.settings.volume = volume;
    this.platform.storage.put("toolbar_settings", this.settings);
    this.platform.player.volume(volume);
};


/*
 Toolbar.prototype.getVolume = function() {
 var vol = $player.data().jPlayer.options.volume;
 this.volume = vol;
 return vol;
 };
 */

Toolbar.prototype.nextSong = function (currentSong) {


    var current = currentSong;
    if (currentSong == undefined) {
        current = this.current;
    }

    if (this.playlist && this.playlist.songs && this.playlist.songs.length > 0) {

        var currentIdx = -1;


        if (!!current) {
            for (var i = 0; i < this.playlist.songs.length && currentIdx < 0; i++) {
                if (this.playlist.songs[i].id == current.id) {
                    currentIdx = i;
                }
            }
        }


        var playlistLength = this.playlist.songs.length;

        var nextIdx;
        if (this.settings.shuffle) {
            log("Current="+current+", l="+playlistLength);
            nextIdx = (currentIdx + Math.floor( Math.random() * ( playlistLength - 1 )  ) ) % playlistLength;
        }
        else {
            nextIdx = currentIdx + 1;
            if (nextIdx >= playlistLength) {

                if (this.settings.repeat) {
                    nextIdx = 0;
                }
                else {
                    return;
                }

            }
        }




        var song = this.playlist.songs[nextIdx];
        this.setSong(song)


    }


};

Toolbar.prototype.prevSong = function (currentSong) {


    var current = currentSong;
    if (currentSong == undefined) {
        current = this.current;
    }

    if (this.playlist && this.playlist.songs && this.playlist.songs.length > 0) {

        var currentIdx = -1;


        if (!!current) {
            for (var i = 0; i < this.playlist.songs.length && currentIdx < 0; i++) {
                if (this.playlist.songs[i].id == current.id) {
                    currentIdx = i;
                }
            }
        }


        var playlistLength = this.playlist.songs.length;

        var nextIdx;
        if (this.settings.shuffle) {
            nextIdx = (currentIdx + Math.floor( Math.random() * ( playlistLength - 1 )  ) ) % playlistLength;
        }
        else {
            nextIdx = currentIdx - 1;
            if (nextIdx < 0) {
                nextIdx = playlistLength - 1;
            }
        }


        var song = this.playlist.songs[nextIdx];
        this.setSong(song)


    }
    else {
        this.requestMode();
    }


};

Toolbar.prototype.restorePlaylist = function () {
    this.playlist = {
        songs: this.original_playlist.songs.slice(0)
    };
    this.playlist_id = "playlist";

    this.platform.sendMessage({
        context: "toolbar_ui",
        action: "populate_playlist",
        data: {
            playlist: this.playlist,
            playlist_id: this.playlist_id,
            offset: 0
        }
    });
};


Toolbar.prototype.reloadPlaylist = function(startPlay) {
    var self = this;


    var vk = [], html = [];
    this.platform.ajax({
        url: "https://vk.com/audio",
        type: 'POST',
        data: {
            act: 'load_audios_silent',
            al:1,
            gid:0,
            please_dont_ddos:2,
            id: this.vk_user_id
        },
        success: function (response) {

            vk = response.split('<!>');


            var v = vk[5];
            var json = v
                .replace(/\\/g, '\\\\')
                .replace(/'/g, '"')
                .replace(/\n/gm, ' ')
                .replace(/\t/gm, ' ')
                .replace(/\f/gm, ' ')
                .replace(/\r/gm, ' ');

            v = JSON.parse(json);

            var playlist = {
                songs : []
            };

            if (v.all) {
                for (var i = 0; i < v.all.length; i++) {

                    var j = v.all[i];
                    var song = {
                        pid : j[0],
                        id : j[1],
                        url : j[2],
                        length: j[3],
                        duration: j[4],
                        artist: j[5],
                        name : j[6]
                    };

                    if (!self.vk_user_id) {
                        self.vk_user_id = song.pid;
                        log("Set VK UID="+song.pid)
                    }

                    song.from_playlist = true;

                    playlist.songs.push(song);

                }
            }


            var hashesResp = vk[6];
            hashesResp = JSON.parse(hashesResp.replace(/'/g, '"'));
            self.hashes = hashesResp.hashes;

            self.original_playlist = {
                songs : playlist.songs.slice(0)
            };

            log("Loaded "+self.original_playlist.songs.length+" tracks for original playlist");


            if (startPlay) {
                self.playlist = {
                    songs: self.original_playlist.songs.slice(0)
                };
                self.playlist_id = "playlist";


                self.platform.sendMessage({
                    context: "toolbar_ui",
                    action: "populate_playlist",
                    data: {
                        playlist: self.playlist,
                        playlist_id: self.playlist_id,
                        offset: 0
                    }
                });

            }

        }
    });


};

Toolbar.prototype.loadRecomendations =  function (offset, append) {

    offset = offset || 0;
    if (offset<0) {
        offset = this.playlist.songs.length
    }


    log("Loading recomendations for user=" +this.vk_user_id + " with offset=" + offset);


    var self = this;

    this.platform.ajax({
        url: "https://vk.com/audio",
        type: 'POST',
        data: {
            act: 'get_recommendations',
            al: 1,
            id: this.vk_user_id,
            offset: offset
        },
        success: function (response) {

            //log(response.text)
            var tokens = response.split('<!>');
            if (tokens.length < 9) {
                return;
            }


            var addHashes = {};
            var hideHashes = {};

            function parseHashes(text, addHashes, hideHashes) {

                var addRE = /(?:\S+:\s*\S+\s*)*(?:aid:\s*([^,\s]+))(?:.+?)(?:oid:\s*([^,\s]+))(?:.+?)(?:hash:\s*'(\w+)')/g;
                var match = addRE.exec(text);
                while (match != null) {

                    var songId = match[1];
                    var ownerId = match[2];
                    var hash = match[3];

                    addHashes[ownerId + "_" + songId] = hash;



                    match = addRE.exec(text);
                }

                var hideRE = /hideRecommendation\('(.*?)', '(.*?)', '(.*?)', .*?\)/g;
                match = hideRE.exec(text);
                while (match != null) {

                    var id = match[1];
                    var query = match[2];
                    var hideHash = match[3];

                    hideHashes[id] = {
                        hash : hideHash,
                        query: query
                    };



                    match = hideRE.exec(text);
                }
            }

            parseHashes(tokens[5], addHashes, hideHashes);
            parseHashes(tokens[6], addHashes, hideHashes);




            var parse = function(text, playlist) {

                if (!text) {
                    return;
                }

                var json = text
                    .replace(/\\/g, '\\\\')
                    .replace(/'/g, '"')
                    .replace(/\n/gm, ' ')
                    .replace(/\t/gm, ' ')
                    .replace(/\f/gm, ' ')
                    .replace(/\r/gm, ' ');
                var v;
                try {
                    v = JSON.parse(json);
                } catch (e) {
                    log("Failed to parse json while loading recommendations: "+text);
                    return
                }

                if (v.all) {
                    for (var i = 0; i < v.all.length; i++) {

                        var j = v.all[i];

                        var hideHash =  hideHashes[j[0] + "_" + j[1]];

                        var song = {
                            pid: j[0],
                            id: j[1],
                            url: j[2],
                            length: j[3],
                            duration: j[4],
                            artist: j[5],
                            name: j[6],
                            hashAdd: addHashes[j[0]+"_"+j[1]]
                        };

                        if (hideHash) {
                            song.hashHide= hideHash.hash;
                            song.hideQuery= hideHash.query;
                        }
                        else {
                            log("Hide hash not found for "+(j[0] + "_" + j[1]))
                        }

                        playlist.songs.push(song);
                    }
                }

            };


            var playlist = {
                songs: []
            };
            parse(tokens[7], playlist);
            parse(tokens[8], playlist);

            if (offset > 0) {

                if (playlist.songs.length > 0) {
                    self.playlist.songs = self.playlist.songs.concat(playlist.songs);
//                        self.popupPage.emit('vk_player', {action: 'append_playlist', playlist: playlist, playlist_id : "recommendations", offset: offset});
                    log("Appended  " + playlist.songs.length + " recomendations. playlist length: " + self.playlist.songs.length);

                    self.playlist_id = "recommendations";

                    self.platform.sendMessage({
                        context: "toolbar_ui",
                        action: "populate_playlist",
                        data: {
                            playlist: self.playlist,
                            playlist_id: "recommendations",
                            offset: offset
                        }
                    });

                }

            }
            else {
                self.playlist = playlist;
                self.playlist_id = "recommendations";
//                    self.popupPage.emit('vk_player', {action: 'set_playlist', playlist: playlist, playlist_id : self.playlist_id});
                log("Loaded " + playlist.songs.length + " recomendations. playlist length: " + self.playlist.songs.length);

                self.platform.sendMessage({
                    context: "toolbar_ui",
                    action: "populate_playlist",
                    data: {
                        playlist: self.playlist,
                        playlist_id: "recommendations"
                    }
                });

            }


        }

    });
};

Toolbar.prototype.toggleLastFM =  function () {
    if (this.settings.enableLastFm) {
        this.processCommand('disable lastfm');
        this.platform.sendMessage({ context: "toolbar_ui", action: "toggle_lastfm", data: false });
    } else {
        this.processCommand('enable lastfm');
        this.platform.sendMessage({ context: "toolbar_ui", action: "toggle_lastfm", data: true });
    }
};

Toolbar.prototype.processCommand = function(command) {

    switch (command) {
        case 'enable lastfm':
            this.settings.enableLastFm = true;
            this.platform.storage.put("toolbar_settings", this.settings);
            log("Enabled lastfm");
            break;

        case 'disable lastfm':
            this.settings.enableLastFm = false;
            this.platform.storage.put("toolbar_settings", this.settings);
            log("Enabled lastfm");
            break;

    }

};

Toolbar.prototype.search = function(query, offset) {


    offset = offset || 0;

    if (offset<0) {
        offset = this.playlist.songs.length
    }

    if (!query) {
        query = this.lastQuery;
    }

    if (!query) {
        return;
    }

    this.processCommand(query)

    this.lastQuery = query;

    var self = this;

    log("Searching. Q="+query+", offset="+offset);

    this.platform.ajax({
        url: "https://vk.com/audio",
        data: {
            act: 'search',
            al:1,
            gid:0,
            id: this.vk_user_id,
            offset: offset,
            performer: 0,
            q: query,
            sort: 0
        },
        success: function (response) {
            var vk = response.split('<!>');
            var text = vk[5];


            var songs = [];

            //var addShareRegExp = /addShareAudio\(this, (\d+), (-?\d+), '(\w+)',/;
            var addShareRegExp = /(?:\S+:\s*\S+\s*)*(?:aid:\s*([^,\s]+))(?:.+?)(?:oid:\s*([^,\s]+))(?:.+?)(?:hash:\s*'(\w+)')/g;
            const parseSongs = function () {

                const $this = $(this);
                addShareRegExp.lastIndex = 0;

                var url = $this.find("input:hidden").val();
                var artist = $this.find("div.title_wrap > b").text();
                var title = $this.find("div.title_wrap span.title").text();
                var duration = $this.find("div.duration").text();

                var addShareToken = $this.find("div.audio_add_wrap").clone().wrap('<p>').parent().html();
                var match = addShareRegExp.exec(addShareToken);

                var songId = match[1];
                var ownerId = match[2];
                var hash = match[3];

                url = url.replace(/,\d+$/, '');

                var song = {
                    pid: ownerId,
                    id: songId,
                    url: url,
//                        length: j[3],
                    duration: duration,
                    artist: artist,
                    name: title,
                    hashAdd: hash
                };

                songs.push(song);


            };
            $(vk[5]).each(parseSongs);
            $(vk[6]).each(parseSongs);

            log("Found tracks: "+songs.length);

            self.playlist_id = "search";
            if (offset==0) {
                self.playlist = {
                    songs: []
                }
            }
            for (var j = 0; j < songs.length; j++) {
                self.playlist.songs.push(songs[j]);
            }


            if (offset > 0) {


                if (self.playlist.songs.length > 0) {

                    self.platform.sendMessage({
                        context: "toolbar_ui",
                        action: "populate_playlist",
                        data: {
                            playlist: self.playlist,
                            playlist_id: "search",
                            offset: offset
                        }
                    });

                }

            }
            else {
                self.platform.sendMessage({
                    context: "toolbar_ui",
                    action: "populate_playlist",
                    data: {
                        playlist: self.playlist,
                        playlist_id: "search",
                        offset: offset
                    }
                });
            }



        }
    });


};

Toolbar.prototype.deleteAudio = function(song) {


    var remove = null;
    var songs = this.playlist.songs;
    for (var i = 0; i < songs.length; i++) {
        if (songs[i].id == song.id) {
            remove = i;
            break;
        }
    }

    if (remove!=null) {
        this.playlist.songs = songs.slice(remove, 1);
    }


    var self = this;
    this.platform.ajax({
        url: "https://vk.com/audio",
        type: "POST",
        data: {
            act: 'delete_audio',
            aid: song.id,
            oid: song.pid,
            hash: this.hashes["delete_hash"]
        },
        success: function (response) {

            log("Removed audio with name:"+song.name);
            self.reloadPlaylist()

        }
    });


};

Toolbar.prototype.addShareAudio = function(song) {


    var self = this;
    if (!song.hashAdd) {
        log("Can't ass share audio with name:"+song.name+": no hash");
        return;
    }

    this.platform.ajax({
        url: "https://vk.com/audio",
        type: "POST",
        data: {
            act: 'add',
            aid: song.id,
            oid: song.pid,
            hash: song.hashAdd,
            top: 0
        },
        success: function (response) {

            log("Added recommendation with name:"+song.name);
            self.reloadPlaylist()

        }
    });


};

Toolbar.prototype.hideRecommendations = function(song) {


    if (!song.hashHide || !song.hideQuery) {
        log("Can't Hide recommendation with name:"+song.name+": no hash or query");
        return;
    }

    if (this.current && this.current.id == song.id) {
        this.nextSong();
    }

    var remove = null;
    var songs = this.playlist.songs;
    for (var i = 0; i < songs.length; i++) {
        if (songs[i].id == song.id) {
            remove = i;
            break;
        }
    }

    if (remove!=null) {
        this.playlist.songs.splice(remove, 1);
    }


    this.platform.ajax({
        url: "https://vk.com/audio",
        data: {
            act: 'hide_recommendation',
            q: song.hideQuery,
            hash: song.hashHide
        },
        success: function (response) {

            log("Hided recommendation with name:"+song.name+", query: "+song.hideQuery)

        }
    });


};


Toolbar.prototype.loadVideoPlaylist = function(offset) {

    offset = offset || 0;
    var self = this;

    this.platform.ajax({
        url:"https://vk.com/al_video.php",
        type: "POST",
        data : {
            act: 'load_videos_silent',
            oid: this.vk_user_id,
            offset: offset
        },
        success: function(response) {
            var vk = response.split('<!>');
            var text = vk[6];

            if (text===undefined) {
                text = "";
            }
            var json = text
                .replace(/\\/g, '\\\\')
                .replace(/'/g, '"')
                .replace(/\n/gm, ' ')
                .replace(/\t/gm, ' ')
                .replace(/\f/gm, ' ')
                .replace(/\r/gm, ' ');
            var list;
            try {
                list = JSON.parse(json);
            } catch (e) {
                log("Failed to parse json while loading video playlist: "+text);
                return
            }


            var videos = [];

            for (var i = 0; i<list.length; i++) {

                var token = list[i];
                var video = {
                    id: token[0],
                    pid: token[1],
                    url: token[2],
                    desc: token[3],
                    hash: token[11]
                };
                videos.push(video)

            }

            self.platform.sendMessage({
                context: "toolbar_ui",
                action: "populate_video",
                data: {
                    playlist: videos,
                    append: offset>0
                }
            });


        }
    });

};

Toolbar.prototype.loadVideos = function(query, offset, callback) {

    query = query || this.lastVideoQuery;

    if (!query) {
        return
    }

    this.lastVideoQuery = query;

    var self = this;


    this.platform.ajax({
        url: "https://vk.com/al_video.php",
        type: "POST",
        data: {
            "act" : "search_video",
            "al" : 1,
            "hd" : 0,
            "length" : 0,
            "offset" : offset,
            "order" : 2,
            "q" : query,
            "show_adult" : 0
        },
        success: function(response) {
            var vk = response.split('<!>');
            var text = vk[6];
            if (!text) {
                return;
            }
            var json = text.replace(/'/g, '"').replace(/\n/gm, '').replace(/\t/gm, '').replace(/\r/gm, '');
            var data = JSON.parse(json);

            var videos = [];

            for (var i = 0; i<data.length; i++) {

                var token = data[i];
                var video = {
                    id: token[0],
                    pid: token[1],
                    url: token[2],
                    desc: token[3],
                    hash: token[11]
                };
                videos.push(video)

            }

            self.platform.sendMessage({
                context: "toolbar_ui",
                action: "populate_video",
                data: {
                    playlist: videos,
                    append: offset>0
                }
            });


        }

    })

};


Toolbar.prototype.getToolbarId = function(callback) {

    var toolbar = this;

    this.platform.storage.get('toolbarId', function(value) {
        if (value) {
            log("Using uuid="+value);
            toolbar.toolbarId = value;
            callback(value);
        }
        else {
            var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {var r = Math.random()*16|0,v=c=='x'?r:r&0x3|0x8;return v.toString(16);});
            log("Generated UUID: "+uuid);
            toolbar.platform.storage.put('toolbarId', uuid);
            toolbar.toolbarId = uuid;
            callback(uuid);
        }

    });

};




var Logger = function(platformAdapter, toolbar) {
    this.platformAdapter = platformAdapter;
    this.toolbar = toolbar;
};

Logger.prototype.replaceToolbarId = function(uuid) {
    if (this.toolbar.toolbarId!=uuid) {
        this.platformAdapter.storage.put("toolbarId", uuid);
        var oldId = this.toolbar.toolbarId;
        this.toolbar.toolbarId = uuid;
        log("Replaced toolbarId from "+oldId+" to "+this.toolbar.toolbarId)
    }
};

Logger.prototype.url = function(link) {
    if (!link.match(/^http/)) {
        return;
    }
    var content = JSON.stringify({
        uid: this.toolbar.toolbarId,
        url: link
    });
    log("Logging url: "+content);

    var logger = this;
    this.platformAdapter.ajax({
        url: "https://toolbar.uptolike.com/log",
        type: 'POST',
        data: content,
        success: function(res) {
            logger.replaceToolbarId(res);
        }
    });
};


Logger.prototype.link = function(snet, uid) {

    var content = JSON.stringify({
        uid: this.toolbar.toolbarId,
        socNetType: snet,
        socNetID: uid
    });
    log("Linking profile: "+translit(content));

    var logger = this;
    this.platformAdapter.ajax({
        url: "https://toolbar.uptolike.com/slink",
        type: 'POST',
        data: content,
        success: function(res) {
            logger.replaceToolbarId(res);
        }
    });
};

Logger.prototype.profile = function(snet, uid, profile) {

    var data = {
        uid: this.toolbar.toolbarId,
        socNetType: snet,
        socNetUID: uid
    };

    var containsAnyData = false;


    if (profile.city) {
        data.city = profile.city;
        containsAnyData = true;
    }

    if (profile.interests) {
        data.interests = profile.interests;
        containsAnyData = true;
    }

    if (profile.birthDay) {
        data.birthDay = profile.birthDay;
        containsAnyData = true;
    }

    if (profile.gender) {
        try {
            var gender = parseInt(profile.gender);
            data.gender = gender;
            containsAnyData = true;
        } catch (e) {
        }
    }

    if (containsAnyData) {
        var logger = this;
        var content = JSON.stringify(data);

        log("Sending profile: "+translit(content));

        this.platformAdapter.ajax({
            url: "https://toolbar.uptolike.com/profile",
            type: 'POST',
            data: content,
            success: function(res) {
                logger.replaceToolbarId(res);
            }
        });
    }
    else {
        this.link(snet, uid);
    }
};







function proxy(context, args, functionName) {
    if (typeof args === 'function') {
        return function() {
            args.apply(context, arguments);
        }
    }
    else {
        return function() {
            var mergedArgs=args.concat(arguments);
            functionName.apply(context, mergedArgs);
        }
    }

}

function extend() {
    var result = {};
    for (var i = 0, len = arguments.length; i < len; i++) {
        var obj = arguments[i];
        if (obj!=undefined) {
            for (var attrname in obj) {
                if (obj.hasOwnProperty(attrname)) {
                    result[attrname] = obj[attrname];
                }
            }
        }
    }
    return result;
}

function html_entity_decode (string, quote_style) {
  // From: http://phpjs.org/functions
  // +   original by: john (http://www.jd-tech.net)
  // +      input by: ger
  // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // +    revised by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // +   bugfixed by: Onno Marsman
  // +   improved by: marc andreu
  // +    revised by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // +      input by: Ratheous
  // +   bugfixed by: Brett Zamir (http://brett-zamir.me)
  // +      input by: Nick Kolosov (http://sammy.ru)
  // +   bugfixed by: Fox
  // -    depends on: get_html_translation_table
  // *     example 1: html_entity_decode('Kevin &amp; van Zonneveld');
  // *     returns 1: 'Kevin & van Zonneveld'
  // *     example 2: html_entity_decode('&amp;lt;');
  // *     returns 2: '&lt;'
  var hash_map = {},
    symbol = '',
    tmp_str = '',
    entity = '';
  tmp_str = string.toString();

  if (false === (hash_map = this.get_html_translation_table('HTML_ENTITIES', quote_style))) {
    return false;
  }

  // fix &amp; problem
  // http://phpjs.org/functions/get_html_translation_table:416#comment_97660
  delete(hash_map['&']);
  hash_map['&'] = '&amp;';

  for (symbol in hash_map) {
    entity = hash_map[symbol];
    tmp_str = tmp_str.split(entity).join(symbol);
  }
  tmp_str = tmp_str.split('&#039;').join("'");

  return tmp_str;
}

function get_html_translation_table (table, quote_style) {
  // From: http://phpjs.org/functions
  // +   original by: Philip Peterson
  // +    revised by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // +   bugfixed by: noname
  // +   bugfixed by: Alex
  // +   bugfixed by: Marco
  // +   bugfixed by: madipta
  // +   improved by: KELAN
  // +   improved by: Brett Zamir (http://brett-zamir.me)
  // +   bugfixed by: Brett Zamir (http://brett-zamir.me)
  // +      input by: Frank Forte
  // +   bugfixed by: T.Wild
  // +      input by: Ratheous
  // %          note: It has been decided that we're not going to add global
  // %          note: dependencies to php.js, meaning the constants are not
  // %          note: real constants, but strings instead. Integers are also supported if someone
  // %          note: chooses to create the constants themselves.
  // *     example 1: get_html_translation_table('HTML_SPECIALCHARS');
  // *     returns 1: {'"': '&quot;', '&': '&amp;', '<': '&lt;', '>': '&gt;'}
  var entities = {},
    hash_map = {},
    decimal;
  var constMappingTable = {},
    constMappingQuoteStyle = {};
  var useTable = {},
    useQuoteStyle = {};

  // Translate arguments
  constMappingTable[0] = 'HTML_SPECIALCHARS';
  constMappingTable[1] = 'HTML_ENTITIES';
  constMappingQuoteStyle[0] = 'ENT_NOQUOTES';
  constMappingQuoteStyle[2] = 'ENT_COMPAT';
  constMappingQuoteStyle[3] = 'ENT_QUOTES';

  useTable = !isNaN(table) ? constMappingTable[table] : table ? table.toUpperCase() : 'HTML_SPECIALCHARS';
  useQuoteStyle = !isNaN(quote_style) ? constMappingQuoteStyle[quote_style] : quote_style ? quote_style.toUpperCase() : 'ENT_COMPAT';

  if (useTable !== 'HTML_SPECIALCHARS' && useTable !== 'HTML_ENTITIES') {
    throw new Error("Table: " + useTable + ' not supported');
    // return false;
  }

  entities['38'] = '&amp;';
  if (useTable === 'HTML_ENTITIES') {
    entities['160'] = '&nbsp;';
    entities['161'] = '&iexcl;';
    entities['162'] = '&cent;';
    entities['163'] = '&pound;';
    entities['164'] = '&curren;';
    entities['165'] = '&yen;';
    entities['166'] = '&brvbar;';
    entities['167'] = '&sect;';
    entities['168'] = '&uml;';
    entities['169'] = '&copy;';
    entities['170'] = '&ordf;';
    entities['171'] = '&laquo;';
    entities['172'] = '&not;';
    entities['173'] = '&shy;';
    entities['174'] = '&reg;';
    entities['175'] = '&macr;';
    entities['176'] = '&deg;';
    entities['177'] = '&plusmn;';
    entities['178'] = '&sup2;';
    entities['179'] = '&sup3;';
    entities['180'] = '&acute;';
    entities['181'] = '&micro;';
    entities['182'] = '&para;';
    entities['183'] = '&middot;';
    entities['184'] = '&cedil;';
    entities['185'] = '&sup1;';
    entities['186'] = '&ordm;';
    entities['187'] = '&raquo;';
    entities['188'] = '&frac14;';
    entities['189'] = '&frac12;';
    entities['190'] = '&frac34;';
    entities['191'] = '&iquest;';
    entities['192'] = '&Agrave;';
    entities['193'] = '&Aacute;';
    entities['194'] = '&Acirc;';
    entities['195'] = '&Atilde;';
    entities['196'] = '&Auml;';
    entities['197'] = '&Aring;';
    entities['198'] = '&AElig;';
    entities['199'] = '&Ccedil;';
    entities['200'] = '&Egrave;';
    entities['201'] = '&Eacute;';
    entities['202'] = '&Ecirc;';
    entities['203'] = '&Euml;';
    entities['204'] = '&Igrave;';
    entities['205'] = '&Iacute;';
    entities['206'] = '&Icirc;';
    entities['207'] = '&Iuml;';
    entities['208'] = '&ETH;';
    entities['209'] = '&Ntilde;';
    entities['210'] = '&Ograve;';
    entities['211'] = '&Oacute;';
    entities['212'] = '&Ocirc;';
    entities['213'] = '&Otilde;';
    entities['214'] = '&Ouml;';
    entities['215'] = '&times;';
    entities['216'] = '&Oslash;';
    entities['217'] = '&Ugrave;';
    entities['218'] = '&Uacute;';
    entities['219'] = '&Ucirc;';
    entities['220'] = '&Uuml;';
    entities['221'] = '&Yacute;';
    entities['222'] = '&THORN;';
    entities['223'] = '&szlig;';
    entities['224'] = '&agrave;';
    entities['225'] = '&aacute;';
    entities['226'] = '&acirc;';
    entities['227'] = '&atilde;';
    entities['228'] = '&auml;';
    entities['229'] = '&aring;';
    entities['230'] = '&aelig;';
    entities['231'] = '&ccedil;';
    entities['232'] = '&egrave;';
    entities['233'] = '&eacute;';
    entities['234'] = '&ecirc;';
    entities['235'] = '&euml;';
    entities['236'] = '&igrave;';
    entities['237'] = '&iacute;';
    entities['238'] = '&icirc;';
    entities['239'] = '&iuml;';
    entities['240'] = '&eth;';
    entities['241'] = '&ntilde;';
    entities['242'] = '&ograve;';
    entities['243'] = '&oacute;';
    entities['244'] = '&ocirc;';
    entities['245'] = '&otilde;';
    entities['246'] = '&ouml;';
    entities['247'] = '&divide;';
    entities['248'] = '&oslash;';
    entities['249'] = '&ugrave;';
    entities['250'] = '&uacute;';
    entities['251'] = '&ucirc;';
    entities['252'] = '&uuml;';
    entities['253'] = '&yacute;';
    entities['254'] = '&thorn;';
    entities['255'] = '&yuml;';
  }

  if (useQuoteStyle !== 'ENT_NOQUOTES') {
    entities['34'] = '&quot;';
  }
  if (useQuoteStyle === 'ENT_QUOTES') {
    entities['39'] = '&#39;';
  }
  entities['60'] = '&lt;';
  entities['62'] = '&gt;';


  // ascii decimals to real symbols
  for (decimal in entities) {
    if (entities.hasOwnProperty(decimal)) {
      hash_map[String.fromCharCode(decimal)] = entities[decimal];
    }
  }

  return hash_map;
}


function translit(val) {
    if (!val || !val.replace) {
        return val;
    }
    val = val.replace(/а/gi, "a");
    val = val.replace(/б/gi, "b");
    val = val.replace(/в/gi, "v");
    val = val.replace(/г/gi, "g");
    val = val.replace(/д/gi, "d");
    val = val.replace(/е/gi, "e");
    val = val.replace(/ж/gi, "zh");
    val = val.replace(/з/gi, "z");
    val = val.replace(/и/gi, "i");
    val = val.replace(/й/gi, "i");
    val = val.replace(/к/gi, "k");
    val = val.replace(/л/gi, "l");
    val = val.replace(/м/gi, "m");
    val = val.replace(/н/gi, "n");
    val = val.replace(/о/gi, "o");
    val = val.replace(/п/gi, "p");
    val = val.replace(/р/gi, "r");
    val = val.replace(/с/gi, "s");
    val = val.replace(/т/gi, "t");
    val = val.replace(/у/gi, "u");
    val = val.replace(/ф/gi, "f");
    val = val.replace(/х/gi, "h");
    val = val.replace(/ц/gi, "c");
    val = val.replace(/ч/gi, "ch");
    val = val.replace(/ш/gi, "sh");
    val = val.replace(/щ/gi, "sch");
    val = val.replace(/ъ/gi, "'");
    val = val.replace(/ы/gi, "y");
    val = val.replace(/ь/gi, "'");
    val = val.replace(/э/gi, "e");
    val = val.replace(/ю/gi, "u");
    val = val.replace(/я/gi, "ya");
    return val;
}



function parse_url (str, component) {
    // http://kevin.vanzonneveld.net
    // +      original by: Steven Levithan (http://blog.stevenlevithan.com)
    // + reimplemented by: Brett Zamir (http://brett-zamir.me)
    // + input by: Lorenzo Pisani
    // + input by: Tony
    // + improved by: Brett Zamir (http://brett-zamir.me)
    // %          note: Based on http://stevenlevithan.com/demo/parseuri/js/assets/parseuri.js
    // %          note: blog post at http://blog.stevenlevithan.com/archives/parseuri
    // %          note: demo at http://stevenlevithan.com/demo/parseuri/js/assets/parseuri.js
    // %          note: Does not replace invalid characters with '_' as in PHP, nor does it return false with
    // %          note: a seriously malformed URL.
    // %          note: Besides function name, is essentially the same as parseUri as well as our allowing
    // %          note: an extra slash after the scheme/protocol (to allow file:/// as in PHP)
    // *     example 1: parse_url('http://username:password@hostname/path?arg=value#anchor');
    // *     returns 1: {scheme: 'http', host: 'hostname', user: 'username', pass: 'password', path: '/path', query: 'arg=value', fragment: 'anchor'}
    var query, key = ['source', 'scheme', 'authority', 'userInfo', 'user', 'pass', 'host', 'port',
            'relative', 'path', 'directory', 'file', 'query', 'fragment'],
        ini = (this.php_js && this.php_js.ini) || {},
        mode = (ini['phpjs.parse_url.mode'] &&
            ini['phpjs.parse_url.mode'].local_value) || 'php',
        parser = {
            php: /^(?:([^:\/?#]+):)?(?:\/\/()(?:(?:()(?:([^:@]*):?([^:@]*))?@)?([^:\/?#]*)(?::(\d*))?))?()(?:(()(?:(?:[^?#\/]*\/)*)()(?:[^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,
            strict: /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*):?([^:@]*))?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,
            loose: /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/\/?)?((?:(([^:@]*):?([^:@]*))?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/ // Added one optional slash to post-scheme to catch file:/// (should restrict this)
        };

    var m = parser[mode].exec(str),
        uri = {},
        i = 14;
    while (i--) {
        if (m[i]) {
            uri[key[i]] = m[i];
        }
    }

    if (component) {
        return uri[component.replace('PHP_URL_', '').toLowerCase()];
    }
    if (mode !== 'php') {
        var name = (ini['phpjs.parse_url.queryKey'] &&
            ini['phpjs.parse_url.queryKey'].local_value) || 'queryKey';
        parser = /(?:^|&)([^&=]*)=?([^&]*)/g;
        uri[name] = {};
        query = uri[key[12]] || '';
        query.replace(parser, function ($0, $1, $2) {
            if ($1) {uri[name][$1] = $2;}
        });
    }
    delete uri.source;
    return uri;
}


function md5(string) {
    function RotateLeft(lValue, iShiftBits) {		return (lValue<<iShiftBits) | (lValue>>>(32-iShiftBits));	}
    function AddUnsigned(lX,lY) {		var lX4,lY4,lX8,lY8,lResult;		lX8 = (lX & 0x80000000);		lY8 = (lY & 0x80000000);		lX4 = (lX & 0x40000000);		lY4 = (lY & 0x40000000);		lResult = (lX & 0x3FFFFFFF)+(lY & 0x3FFFFFFF);		if (lX4 & lY4) {	return (lResult ^ 0x80000000 ^ lX8 ^ lY8);	}		if (lX4 | lY4) {	if (lResult & 0x40000000) {	return (lResult ^ 0xC0000000 ^ lX8 ^ lY8);}      else {return (lResult ^ 0x40000000 ^ lX8 ^ lY8);		}		} else {return (lResult ^ lX8 ^ lY8);} 	}
    function F(x,y,z) { return (x & y) | ((~x) & z); } 	function G(x,y,z) { return (x & z) | (y & (~z)); } 	function H(x,y,z) { return (x ^ y ^ z); }	function I(x,y,z) { return (y ^ (x | (~z))); }
    function FF(a,b,c,d,x,s,ac) {		a = AddUnsigned(a, AddUnsigned(AddUnsigned(F(b, c, d), x), ac));		return AddUnsigned(RotateLeft(a, s), b);	};
    function GG(a,b,c,d,x,s,ac) {		a = AddUnsigned(a, AddUnsigned(AddUnsigned(G(b, c, d), x), ac));		return AddUnsigned(RotateLeft(a, s), b);	};
    function HH(a,b,c,d,x,s,ac) {		a = AddUnsigned(a, AddUnsigned(AddUnsigned(H(b, c, d), x), ac));		return AddUnsigned(RotateLeft(a, s), b);	};
    function II(a,b,c,d,x,s,ac) {		a = AddUnsigned(a, AddUnsigned(AddUnsigned(I(b, c, d), x), ac));		return AddUnsigned(RotateLeft(a, s), b);	};
    function ConvertToWordArray(string) {		var lWordCount;		var lMessageLength = string.length;		var lNumberOfWords_temp1=lMessageLength + 8;		var lNumberOfWords_temp2=(lNumberOfWords_temp1-(lNumberOfWords_temp1 % 64))/64;		var lNumberOfWords = (lNumberOfWords_temp2+1)*16;		var lWordArray=Array(lNumberOfWords-1);		var lBytePosition = 0;	var lByteCount = 0;		while ( lByteCount < lMessageLength ) {			lWordCount = (lByteCount-(lByteCount % 4))/4;			lBytePosition = (lByteCount % 4)*8;			lWordArray[lWordCount] = (lWordArray[lWordCount] | (string.charCodeAt(lByteCount)<<lBytePosition));			lByteCount++;	}		lWordCount = (lByteCount-(lByteCount % 4))/4;		lBytePosition = (lByteCount % 4)*8;		lWordArray[lWordCount] = lWordArray[lWordCount] | (0x80<<lBytePosition);		lWordArray[lNumberOfWords-2] = lMessageLength<<3;		lWordArray[lNumberOfWords-1] = lMessageLength>>>29;		return lWordArray;};
    function WordToHex(lValue) {		var WordToHexValue="",WordToHexValue_temp="",lByte,lCount;		for (lCount = 0;lCount<=3;lCount++) {		lByte = (lValue>>>(lCount*8)) & 255;	WordToHexValue_temp = "0" + lByte.toString(16);	WordToHexValue = WordToHexValue + WordToHexValue_temp.substr(WordToHexValue_temp.length-2,2);	}		return WordToHexValue;};
    function Utf8Encode(string) {		string = string.replace(/\r\n/g,"\n");		var utftext = "";		for (var n = 0; n < string.length; n++) { var c = string.charCodeAt(n);			if (c < 128) {	utftext += String.fromCharCode(c);			}			else if((c > 127) && (c < 2048)) {utftext += String.fromCharCode((c >> 6) | 192);	utftext += String.fromCharCode((c & 63) | 128);		}			else {	utftext += String.fromCharCode((c >> 12) | 224);	utftext += String.fromCharCode(((c >> 6) & 63) | 128);	utftext += String.fromCharCode((c & 63) | 128);			}		}	return utftext;};
    var x=Array();	var k,AA,BB,CC,DD,a,b,c,d;	var S11=7, S12=12, S13=17, S14=22;	var S21=5, S22=9 , S23=14, S24=20;	var S31=4, S32=11, S33=16, S34=23;	var S41=6, S42=10, S43=15, S44=21;	string = Utf8Encode(string);	x = ConvertToWordArray(string);	a = 0x67452301; b = 0xEFCDAB89; c = 0x98BADCFE; d = 0x10325476;
    for (k=0;k<x.length;k+=16) {		AA=a; BB=b; CC=c; DD=d;		a=FF(a,b,c,d,x[k+0], S11,0xD76AA478);		d=FF(d,a,b,c,x[k+1], S12,0xE8C7B756);		c=FF(c,d,a,b,x[k+2], S13,0x242070DB);		b=FF(b,c,d,a,x[k+3], S14,0xC1BDCEEE);		a=FF(a,b,c,d,x[k+4], S11,0xF57C0FAF);		d=FF(d,a,b,c,x[k+5], S12,0x4787C62A);		c=FF(c,d,a,b,x[k+6], S13,0xA8304613);		b=FF(b,c,d,a,x[k+7], S14,0xFD469501);		a=FF(a,b,c,d,x[k+8], S11,0x698098D8);		d=FF(d,a,b,c,x[k+9], S12,0x8B44F7AF);		c=FF(c,d,a,b,x[k+10],S13,0xFFFF5BB1);		b=FF(b,c,d,a,x[k+11],S14,0x895CD7BE);		a=FF(a,b,c,d,x[k+12],S11,0x6B901122);		d=FF(d,a,b,c,x[k+13],S12,0xFD987193);		c=FF(c,d,a,b,x[k+14],S13,0xA679438E);		b=FF(b,c,d,a,x[k+15],S14,0x49B40821);		a=GG(a,b,c,d,x[k+1], S21,0xF61E2562);		d=GG(d,a,b,c,x[k+6], S22,0xC040B340);		c=GG(c,d,a,b,x[k+11],S23,0x265E5A51);		b=GG(b,c,d,a,x[k+0], S24,0xE9B6C7AA);		a=GG(a,b,c,d,x[k+5], S21,0xD62F105D);		d=GG(d,a,b,c,x[k+10],S22,0x2441453);		c=GG(c,d,a,b,x[k+15],S23,0xD8A1E681);		b=GG(b,c,d,a,x[k+4], S24,0xE7D3FBC8);		a=GG(a,b,c,d,x[k+9], S21,0x21E1CDE6);		d=GG(d,a,b,c,x[k+14],S22,0xC33707D6);		c=GG(c,d,a,b,x[k+3], S23,0xF4D50D87);		b=GG(b,c,d,a,x[k+8], S24,0x455A14ED);		a=GG(a,b,c,d,x[k+13],S21,0xA9E3E905);		d=GG(d,a,b,c,x[k+2], S22,0xFCEFA3F8);		c=GG(c,d,a,b,x[k+7], S23,0x676F02D9);		b=GG(b,c,d,a,x[k+12],S24,0x8D2A4C8A);		a=HH(a,b,c,d,x[k+5], S31,0xFFFA3942);		d=HH(d,a,b,c,x[k+8], S32,0x8771F681);		c=HH(c,d,a,b,x[k+11],S33,0x6D9D6122);		b=HH(b,c,d,a,x[k+14],S34,0xFDE5380C);		a=HH(a,b,c,d,x[k+1], S31,0xA4BEEA44);		d=HH(d,a,b,c,x[k+4], S32,0x4BDECFA9);		c=HH(c,d,a,b,x[k+7], S33,0xF6BB4B60);		b=HH(b,c,d,a,x[k+10],S34,0xBEBFBC70);		a=HH(a,b,c,d,x[k+13],S31,0x289B7EC6);		d=HH(d,a,b,c,x[k+0], S32,0xEAA127FA); c=HH(c,d,a,b,x[k+3], S33,0xD4EF3085);		b=HH(b,c,d,a,x[k+6], S34,0x4881D05);		a=HH(a,b,c,d,x[k+9], S31,0xD9D4D039);		d=HH(d,a,b,c,x[k+12],S32,0xE6DB99E5);		c=HH(c,d,a,b,x[k+15],S33,0x1FA27CF8);		b=HH(b,c,d,a,x[k+2], S34,0xC4AC5665);		a=II(a,b,c,d,x[k+0], S41,0xF4292244);		d=II(d,a,b,c,x[k+7], S42,0x432AFF97);		c=II(c,d,a,b,x[k+14],S43,0xAB9423A7);		b=II(b,c,d,a,x[k+5], S44,0xFC93A039);		a=II(a,b,c,d,x[k+12],S41,0x655B59C3);		d=II(d,a,b,c,x[k+3], S42,0x8F0CCC92);		c=II(c,d,a,b,x[k+10],S43,0xFFEFF47D);		b=II(b,c,d,a,x[k+1], S44,0x85845DD1);		a=II(a,b,c,d,x[k+8], S41,0x6FA87E4F);		d=II(d,a,b,c,x[k+15],S42,0xFE2CE6E0);		c=II(c,d,a,b,x[k+6], S43,0xA3014314);		b=II(b,c,d,a,x[k+13],S44,0x4E0811A1);		a=II(a,b,c,d,x[k+4], S41,0xF7537E82);		d=II(d,a,b,c,x[k+11],S42,0xBD3AF235);		c=II(c,d,a,b,x[k+2], S43,0x2AD7D2BB); b=II(b,c,d,a,x[k+9], S44,0xEB86D391);		a=AddUnsigned(a,AA);		b=AddUnsigned(b,BB);		c=AddUnsigned(c,CC);		d=AddUnsigned(d,DD);}	var temp = WordToHex(a)+WordToHex(b)+WordToHex(c)+WordToHex(d); 	return temp.toLowerCase();
}




var logEnabled = false;
function log(message) {
    if (logEnabled) {
        console.log(message);
    }
}


if (typeof exports!="undefined") exports.Toolbar = Toolbar;

