// import the modules we need
var tabs = require("sdk/tabs");
var data = require("sdk/self").data;
var {Cc, Ci, components, Cu} = require('chrome');
var mediator = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator);
var pageMod = require("sdk/page-mod");
var ToolbarLib = require("./Toolbar");
var ObjectRef = require("./object_ref");
var { Class } = require('sdk/core/heritage');
var ss = require("sdk/simple-storage");
var { open, openDialog } = require('sdk/window/utils');
var { setInterval } = require('sdk/timers')
var Request = require("sdk/request").Request;

var logEnabled = false;
function log(message) {
    if (logEnabled) {
        console.log(message);
    }
}


var mp3Player = ObjectRef.ObjectRef();

var popupPage = Class({

    extends: ObjectRef.ObjectRef,

    log : function(msg) {
        this.emit("log", {msg:msg});
    }

})();

var toolbarId;
var authLastFM;
ss.storage.history = ss.storage.history || [];
//ss.storage.last_fm_token = undefined;
//ss.storage.last_fm_session_key = undefined;

var firefoxAdapter = {

    sendMessage: function (msg) {
        popupPage.emit(msg.context, {action: msg.action, data: msg.data });
    },

    ajax: function (params) {
        var type = params.type || "POST";

        var req = Request({
            url: params.url,
            content: params.data,
            onComplete: function (response) {
                params.success(params.url === 'http://ws.audioscrobbler.com/2.0/' &&
                    response.json ? response.json : response.text);
            }
        });

        if (type == "POST") {
            req.post();
        }
        else {
            req.get();
        }
    },

    openWindow: function (params, cb) {
        if (authLastFM) {
            return;
        }
        authLastFM = true;
        let windows = require('sdk/windows').browserWindows
        log("Opening " + params.url)
        let some = windows.open({
            url: params.url,
            onOpen: function (window) {
                var wind = require('sdk/window/utils').getMostRecentBrowserWindow()
                    , wid = wind.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIDOMWindowUtils).currentInnerWindowID;

                wind.resizeTo(params.width, params.height);
                wind.moveTo((wind.screen.width - params.width) / 2, (wind.screen.height - params.height) / 2);

                function myObserver()
                {
                  this.register();
                }

                myObserver.prototype = {
                    observe: function(subject, topic) {
                        var curWid
                            , aVisitor
                            , myNsIHttpHeaderVisitor;
                        if (topic === 'inner-window-destroyed') {
                            curWid = subject.QueryInterface(Ci.nsISupportsPRUint64).data;
                            if (wid === curWid) {
                                authLastFM = false;
                                this.unregister();
                            }
                        } else {
                            subject.QueryInterface(Ci.nsIHttpChannel);
                            if (['http://www.last.fm/api/grantaccess', 'http://www.lastfm.ru/api/grantaccess'].indexOf(subject.name) !== -1) {
                                curWid = require('sdk/window/utils').getMostRecentBrowserWindow()
                                    .QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIDOMWindowUtils).currentInnerWindowID;
                                if (curWid !== wid) {
                                    return;
                                }
                                myNsIHttpHeaderVisitor = function () {
                                    this.check = false;
                                };
                                myNsIHttpHeaderVisitor.prototype.visitHeader = function (aHeader, aValue) {
                                    if (aHeader.toLowerCase().indexOf("location") !== -1) {
                                        if (aValue.indexOf("token") !== -1) {
                                            toolbar.lastFM.notify(aValue);
                                        }
                                    }
                                };
                                myNsIHttpHeaderVisitor.prototype.checkTrue = function () {
                                    return this.check;
                                };
                                aVisitor = new myNsIHttpHeaderVisitor();
                                subject.visitResponseHeaders(aVisitor);
                                if (!aVisitor.checkTrue()) {
                                    return;
                                }
                                if (subject.responseStatus === 200) {
                                    authLastFM = false;
                                    this.unregister();
                                }
                            }
                        }
                    },
                    register: function() {
                        var observerService = Cc["@mozilla.org/observer-service;1"]
                                              .getService(Ci.nsIObserverService);
                        observerService.addObserver(this, "http-on-examine-response", false);
                        observerService.addObserver(this, "inner-window-destroyed", false);
                    },
                    unregister: function() {
                        var observerService = Cc["@mozilla.org/observer-service;1"]
                                                .getService(Ci.nsIObserverService);
                        observerService.removeObserver(this, "http-on-examine-response");
                        observerService.removeObserver(this, "inner-window-destroyed");
                    }
                };

                var observer = new myObserver();
                cb(window);
            }
        });
    },

    getURL: function (file) {
        return data.url(file);
    },

    download: function (song) {


        (function () {

            function getDownloadFile(defaultString, fileType) {
                var nsIFilePicker = Ci.nsIFilePicker;

                var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
                var window = require("sdk/window/utils").getMostRecentBrowserWindow();
                fp.init(window, null, nsIFilePicker.modeSave);
                try {
                    var urlExt = defaultString.substr(defaultString.lastIndexOf(".") + 1, 3);
                    if (urlExt != fileType) defaultString += "." + fileType
                } catch (ex) {
                }

                fp.defaultString = defaultString;

                fp.appendFilter(fileType, "*." + fileType);
                var rv = fp.show();
                if (rv == nsIFilePicker.returnOK || rv == nsIFilePicker.returnReplace) {
                    var file = fp.file;
                    var path = fp.file.path;
                    return file;
                }
                return null;
            }

            let uri = Cc["@mozilla.org/network/io-service;1"]
                .getService(Ci.nsIIOService)
                .newURI(song.url, null, null);


            let targetFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);

            const fileName = removeSymbols(song.artist + " - " + song.name);
            const file = getDownloadFile(fileName, "mp3");
            if (!file) {
                return;
            }

            targetFile.initWithPath(file.path);

            let ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
            let fileURI = ioService.newFileURI(file);

            let persist = Cc['@mozilla.org/embedding/browser/nsWebBrowserPersist;1'].createInstance(Ci.nsIWebBrowserPersist);

            let xfer = Cc["@mozilla.org/transfer;1"].createInstance(Ci.nsITransfer);
            xfer.init(uri, fileURI, fileName, null, new Date(), null, persist, false);


            persist.persistFlags = persist.PERSIST_FLAGS_FROM_CACHE | persist.PERSIST_FLAGS_REPLACE_EXISTING_FILES;
            persist.progressListener = xfer;
            persist.saveURI(uri, null, null, null, "", targetFile, null);
        })();


    },

    downloadAll: function (songs) {
        Cu.import("resource://gre/modules/PrivateBrowsingUtils.jsm");
        Cu.import("resource://gre/modules/Services.jsm");

        function getDownloadDir() {
            var nsIFilePicker = Ci.nsIFilePicker;

            var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
            var window = require("sdk/window/utils").getMostRecentBrowserWindow();
            fp.init(window, null, nsIFilePicker.modeGetFolder);
            var rv = fp.show();

            if (rv == nsIFilePicker.returnOK || rv == nsIFilePicker.returnReplace) {
                var file = fp.file;
                var path = fp.file.path;
                return file;
            }
            return null;
        }

        const dir = getDownloadDir();
        if (!dir) {
            return;
        }


        for (var i = 0; i < songs.length; i++) {
            const song = songs[i];

            const uri = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService).newURI(song.url, null, null);
            const targetFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);


            const fileName = removeSymbols(song.artist + " - " + song.name);
            const targetPath = dir.path;
            targetFile.initWithPath(targetPath);
            targetFile.append(fileName + ".mp3");

            const ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
            const fileURI = ioService.newFileURI(targetFile);

            const persist = Cc['@mozilla.org/embedding/browser/nsWebBrowserPersist;1'].createInstance(Ci.nsIWebBrowserPersist);

            const xfer = Cc["@mozilla.org/transfer;1"].createInstance(Ci.nsITransfer);
            xfer.init(uri, fileURI, fileName, null, new Date(), null, persist, false);
            log("Saving url=" + song.url + " to " + targetPath + " -> " + fileName + ".mp3")


            persist.persistFlags = persist.PERSIST_FLAGS_FROM_CACHE | persist.PERSIST_FLAGS_REPLACE_EXISTING_FILES;
            persist.progressListener = xfer;
            persist.saveURI(uri, null, null, null, "", targetFile, null);


        }


    },

    prepareShare: function (type, data, id) {
    
        if (type !== 'post status') {
            return;
        }
        var wid = require('sdk/window/utils').getMostRecentBrowserWindow()
            .QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIDOMWindowUtils).currentInnerWindowID;
        data.shareSuccess = data.shareSuccess.replace(/\*$/, '');

        function myObserver()
        {
          this.register();
        }

        myObserver.prototype = {
            observe: function(subject, topic) {
                var curWid
                    , aVisitor
                    , myNsIHttpHeaderVisitor;
                if (topic === 'inner-window-destroyed') {
                    curWid = subject.QueryInterface(Ci.nsISupportsPRUint64).data;
                    wid === curWid && this.unregister();
                } else {
                    subject.QueryInterface(Ci.nsIHttpChannel);
                    if (subject.name.indexOf(data.shareSuccess) === 0 ||
                        (data.tabInfo[3] === 'lj' && /http:\/\/[a-z1-9_]+\.livejournal\.com\/[1-9]+\.html/i.test(subject.name)) ||
                        (data.tabInfo[3] === 'su' && /https?:\/\/www.stumbleupon.com\/submit/i.test(subject.name))) {
                        curWid = require('sdk/window/utils').getMostRecentBrowserWindow()
                            .QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIDOMWindowUtils).currentInnerWindowID;
                        if (curWid !== wid) {
                            return;
                        }
                        if (['su', 'bd', 'vk', 'sb'].indexOf(data.tabInfo[3]) !== -1) {
                            if (subject.requestMethod !== 'POST') {
                                return;
                            }
                        }
                        if (data.tabInfo[3] === 'ip' && data.shareSuccess !== subject.name) {
                            return;
                        }
                        if (data.tabInfo[3] === 'bd') {
                            myNsIHttpHeaderVisitor = function () {
                                this.check = false;
                            };
                            myNsIHttpHeaderVisitor.prototype.visitHeader = function (aHeader, aValue) {
                                if (aHeader.toLowerCase().indexOf("set-cookie") !== -1) {
                                    if (aValue.indexOf("sessionid") !== -1) {
                                        this.check = true;
                                    }
                                }
                            };
                            myNsIHttpHeaderVisitor.prototype.checkTrue = function () {
                                return this.check;
                            };
                            aVisitor = new myNsIHttpHeaderVisitor();
                            subject.visitResponseHeaders(aVisitor);
                            if (!aVisitor.checkTrue()) {
                                return;
                            }
                        }
                        if (subject.responseStatus === 200) {
                            this.unregister();
                            firefoxAdapter.addHistoryItem(data.tabInfo);
                        }
                    }
                }
            },
            register: function() {
                var observerService = Cc["@mozilla.org/observer-service;1"]
                                      .getService(Ci.nsIObserverService);
                observerService.addObserver(this, "http-on-examine-response", false);
                observerService.addObserver(this, "inner-window-destroyed", false);
            },
            unregister: function() {
                var observerService = Cc["@mozilla.org/observer-service;1"]
                                        .getService(Ci.nsIObserverService);
                observerService.removeObserver(this, "http-on-examine-response");
                observerService.removeObserver(this, "inner-window-destroyed");
            }
        };

        var observer = new myObserver();
    },

    getHistory: function (memo, i) {
        memo = memo.concat(ss.storage.history);
        toolbar.platform.sendMessage({ context: 'toolbar_ui', action: 'populate_history', data: memo });
    },

    addHistoryItem: function (item) {
        try {
            ss.storage.history.push(item);
        } catch (e) {
            log('error hist ' + e);
        }
        if (ss.quotaUsage > 1) {
           historyFreeSpace(); 
        }
        firefoxAdapter.sendMessage({ context: 'toolbar_ui', action: 'add_history_item', data: item });
    },

    storage: {
        put: function (key, value) {
            ss.storage[key] = value;
        },

        get: function (key, callback) {
            callback(ss.storage[key]);
        }
    },


    player: {
        play: function (pos) {
            mp3Player.emit('vk_player', { action: 'play' });
        },
        seek: function (pos) {
            mp3Player.emit('vk_player', { action: 'seek', pos: pos });
        },
        pause: function () {
            mp3Player.emit('vk_player', { action: 'pause' });
        },
        volume: function (volume) {
            mp3Player.emit('vk_player', {action: 'set_volume', volume: volume});
        },
        setSong: function (song) {
            mp3Player.emit('vk_player', { action: 'set', id: song.pid + "_" + song.id, song: song});
        }
    },

    closeWindow: function (window) {
        authLastFM = false;
        window.close();
    }

};

var toolbar;

firefoxAdapter.storage.get("toolbar_settings", function(settings) {

    toolbar = new ToolbarLib.Toolbar(firefoxAdapter, settings);

    toolbar.search = function(query, offset) {


        offset = offset || 0;

        if (offset<0) {
            offset = this.playlist.songs.length
        }

        if (!query) {
            query = this.lastQuery;
        }

        if (!query) {
            return;
        }

        this.lastQuery = query;

        var toolbar = this;

        log("Searching. Q="+query+", offset="+offset);

        this.platform.ajax({
            url: "https://vk.com/audio",
            data: {
                act: 'search',
                al:1,
                gid:0,
                id: this.vk_user_id,
                offset: offset,
                performer: 0,
                q: query,
                sort: 0
            },
            success: function (response) {
                var vk = response.split('<!>');

                popupPage.getRef().port.once("parsed_search_results", function(songs) {
                    log("Found tracks: "+songs.length);

                    toolbar.playlist_id = "search";
                    if (offset==0) {
                        toolbar.playlist = {
                            songs: []
                        }
                    }
                    for (var j = 0; j < songs.length; j++) {
                        toolbar.playlist.songs.push(songs[j]);
                    }


                    if (offset > 0) {


                        if (toolbar.playlist.songs.length > 0) {

                            toolbar.platform.sendMessage({
                                context: "toolbar_ui",
                                action: "populate_playlist",
                                data: {
                                    playlist: toolbar.playlist,
                                    playlist_id: "search",
                                    offset: offset
                                }
                            });

                        }

                    }
                    else {
                        toolbar.platform.sendMessage({
                            context: "toolbar_ui",
                            action: "populate_playlist",
                            data: {
                                playlist: toolbar.playlist,
                                playlist_id: "search",
                                offset: offset
                            }
                        });
                    }

                });
                popupPage.emit("parse_search_results", [vk[5], vk[6]]);


            }
        });


    };

    toolbar.getToolbarId(function(id) {
        toolbarId = id;
    });

    checkVK();


    var popupPanel = require("sdk/panel").Panel({
        width: 600,
        height: 292,
        contentScriptFile: [
            data.url("common/jquery-2.0.3.min.js"),
            data.url("common/jquery.scrollTo.js"),
            data.url("popup/jquery-ui-1.10.3.custom.min.js"),
            data.url("popup/share.js"),
            data.url("popup/ToolbarUI.js"),
            data.url("popup/popup.js")
        ],
        contentURL: data.url("popup/popup.html")
    });

    popupPanel.port.on('get_share_title', function () {
        popupPanel.port.emit('take_title', tabs.activeTab.title);
    });

    popupPanel.port.on("vk_player", function(msg) {
        toolbar.listener({context: "vk_player", action: msg.action, data: msg.data});
    })
    popupPanel.port.on("model", function (cmd) {
        ss.storage.initialized = true;
        log("Marked as initialized");
    });

    popupPanel.port.on("share", function (cmd) {
        var faviconService = Cc["@mozilla.org/browser/favicon-service;1"].getService(Ci.nsIFaviconService)
            , ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
        switch (cmd.action) {
            case 'request_url' :
                var active = tabs.activeTab.url;
                faviconService.getFaviconDataForPage(ioService.newURI(active, null, null), function (data) {
                    try {
                        popupPage.emit("request_current_url", {
                            data: {
                                url: active
                                , tabInfo: {
                                    favIconUrl: data && data.asciiSpec
                                    , title: tabs.activeTab.title
                                }
                            }
                        });
                    } catch (e) {
                        log(e);
                    }
                });
                break;

            case 'post' :

                var data = cmd.data;
                popupPanel.hide()


                if(data.cmd === 'post') {
                    popupPanel.port.emit('share_init_message_received');
                    log("Opening share dialog: "+data.url + " ["+data.width+"x"+data.height+"]")

                    let windows = require('sdk/windows').browserWindows;

                    let some = windows.open({
                        url: data.url,
                        onOpen: function (window) {
                            var uts = require('sdk/window/utils'),
                                wind = uts.getMostRecentBrowserWindow();
                            exports.main();
                            wind.resizeTo(data.width, data.height);
                            wind.moveTo((wind.screen.width - data.width) / 2, (wind.screen.height - data.height) / 2);
                        }
                    });
                }
                else if(data.cmd === 'mail') {
                    log("mail")
                    tabs.activeTab.url=data.url
                }
                else if(data.cmd === 'print') {
                    log("printing page")
                    tabs.activeTab.url=data.url
                }

                break;
        }
    });

    popupPage.assign(popupPanel);

});








require("sdk/tabs").on("ready", logURL);

function logURL(tab) {
    toolbar.logger.url(tab.url)
}



// exports.main is called when extension is installed or re-enabled
exports.main = function (options, callbacks) {
    var document = mediator.getMostRecentWindow('navigator:browser').document;
    var navBar = document.getElementById('nav-bar');
    if (!navBar) {
        return;
    }

    var btn = document.createElement('toolbarbutton');
    btn.setAttribute('id', 'uptolike-toolbar-trigger');
    btn.setAttribute('type', 'button');
    // the toolbarbutton-1 class makes it look like a traditional button
    btn.setAttribute('class', 'toolbarbutton-1');
    // the data.url is relative to the data folder
    btn.setAttribute('image', data.url('img/logo16.png'));
    btn.setAttribute('width', "28");
    btn.setAttribute('height', "28");
    btn.setAttribute('orient', 'horizontal');
    // this text will be shown when the toolbar is set to text or text and iconss
    btn.setAttribute('label', 'UpToLike!');
    btn.addEventListener('click', function () { if (popupPage.assigned()) popupPage.getRef().show(btn); }, false);
    navBar.appendChild(btn);

};

// exports.onUnload is called when Firefox starts and when the extension is disabled or uninstalled
exports.onUnload = function (reason) {
    require('./install_toolbar_btn').removeToolbarButton();
};



var snIds = {}


/*
// replaced by generic parser
pageMod.PageMod({
    include: "*.facebook.com",
    contentScriptFile: [data.url('common/jquery.js'), data.url('fb-parser.js')],
    onAttach: function(worker) {
        worker.port.on("fb", function(msg) {
            logger.link("FB", msg.id)
            snIds["fb"] = msg.id
        });
    }

});
*/

function historyFreeSpace() {
    ss.storage.history.splice(0, 25);
}

var vkParser = pageMod.PageMod({
    include: ["*.vk.com", "*.vkontakte.ru"],
    contentScriptFile: [data.url('common/jquery-2.0.3.min.js'), data.url('vk-parser.js')],
    onAttach: function(worker) {

        worker.port.on("vk_id", function(msg) {

            toolbar.logger.link("VK", msg.id);
            //if (!toolbar.authenticated) {
                toolbar.setVKUserId(msg.id);
                checkVK();
                snIds["vk"] = msg.id;

            //}


        });

    }
});


var rulesAlgo = ss.storage.parser_rules;
if (!rulesAlgo) {
    var rules = data.load("parsers.json");
    rulesAlgo = JSON.parse(rules);
    ss.storage.parser_rules = rulesAlgo;
    log("Loaded parser rules from supplied file");
}
else {
    log("Loaded parser rules from storage");
}

setInterval(function() {
    var rnd = Math.random();
    Request({
        url: "https://toolbar.uptolike.com/static/parsers.json?r="+rnd,
        onComplete: function (response) {
            rulesAlgo = JSON.parse(response.text);
            ss.storage.parser_rules = rulesAlgo;
            log("Loaded parser rules from server");
        }
    }).get();

}, 1000 * 60 * 60);


for (var snetId in rulesAlgo) {
    if (rulesAlgo.hasOwnProperty(snetId)) {
        var rule = rulesAlgo[snetId];

        log("Creating parser for "+snetId+". Intercepts: "+rule["intercept"])

        pageMod.PageMod({
            include: rule["intercept"],
            contentScriptFile: [data.url('common/jquery-2.0.3.min.js'),data.url('parser.js')],
            contentScriptOptions: {
                snetId: snetId,
                rules: rule

            },
            onAttach: function(worker) {

                worker.port.on("parser", function(msg) {
                    if (msg.profile && msg.profile.uid) {
                        toolbar.logger.profile(msg.snetId, msg.profile.uid, msg.profile)
                    }
                });

            }
        })

    }
}





function checkVK() {
    var vk = [], html = [];
    Request({
        url: "https://vk.com/pads.php",
        content: {
            act: 'pad',
            al: 1,
            init: 1,
            pad_id: 'mus',
            playlist: 1
        },
        onComplete: function (response) {

            vk = response.text.split('<!>');

            if (!vk[7]) {
                log("Not logged into VK")
                popupPage.log("Not logged into VK");
                if (mp3Player.assigned()) {
                    mp3Player.getRef().destroy();
                }
                mp3Player.assign(null);
                toolbar.setAuthenticated(false);
                popupPage.emit("toolbar_ui", {action: "set_authenticated", data: false} );
            }
            else {


                if (!mp3Player.assigned()) {
                    mp3Player.assign(createMp3PlayerPage());
                }

                toolbar.setAuthenticated(true);
                popupPage.emit("toolbar_ui", {action: "set_authenticated", data: true} );
                toolbar.reloadPlaylist(true);

            }
        }
    }).post();

}



function createMp3PlayerPage() {

    var mp3Player = require("sdk/page-worker").Page({
        contentScriptFile: [
            data.url("common/jquery-2.0.3.min.js"),
            data.url("background/background.js")
        ],
        contentURL: "https://vk.com/audio",
        onMessage:function(msg) {
            log("Got message from bkg page: "+msg);
        }
    });

    mp3Player.port.on("log", function(cmd) {
        popupPage.log(cmd.msg)
    });
    mp3Player.port.on("vk_player", function(cmd) {

        switch (cmd.action) {
            case "onPlayProgress" :
                popupPage.emit("toolbar_ui", { action: "update_played", data: cmd.data} );
                break;

            case "onPlayFinish" :
                toolbar.songEnded()
                break;

            case "set_mode" :
                popupPage.emit("toolbar_ui", { action: "set_mode", data: { mode: cmd.mode} } );
                break;

            default :
                popupPage.emit("vk_player", cmd)
                break;

        }

    });


    return mp3Player;
}




var removeSymbols = (function() {

    function removeSymbols (str) {
        if(str && typeof str === 'string') {
            str = str.replace(/[/()\\&;.,#'?<>*:|]/g, "_")
        }

        return str;
    }

    return removeSymbols;
})();
